"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packager = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const fs_1 = require("fs");
const archiver_1 = tslib_1.__importDefault(require("archiver"));
const glob_1 = require("glob");
const rxjs_1 = require("rxjs");
const logger_1 = require("@microlambda/logger");
const execa_1 = require("execa");
const project_1 = require("../graph/project");
const utils_1 = require("@microlambda/utils");
class Packager {
    _useLayers;
    _buildLayer;
    _projectRoot;
    _packagePath;
    _tmpPath;
    _logger;
    constructor(_useLayers = false, _buildLayer = true) {
        this._useLayers = _useLayers;
        this._buildLayer = _buildLayer;
        this._logger = new logger_1.EventsLog().scope('packagr');
        this._projectRoot = (0, utils_1.resolveProjectRoot)();
        this._logger.debug('Initialized packagr on project', this._projectRoot);
    }
    static readMetadata(service) {
        try {
            return (0, fs_extra_1.readJSONSync)((0, path_1.join)(service.root, '.package', 'bundle-metadata.json'));
        }
        catch (e) {
            return { took: 0, megabytes: { code: 0 } };
        }
    }
    bundle(service, level = 4) {
        return new rxjs_1.Observable((obs) => {
            this._logger.debug('Requested to package service', service, '(compression level', level, ')');
            const pkg = async () => {
                const started = Date.now();
                const shouldResolvesServiceNodeModules = !this._useLayers || this._buildLayer;
                let now = Date.now();
                this._logger.debug('Analysing yarn workspaces...');
                const originalProject = await project_1.Project.loadProject(this._projectRoot);
                this._logger.debug('Found', originalProject.workspaces.size, 'workspaces');
                this._logger.debug('Workspaces:', Array.from(originalProject.services.keys()));
                const toPackageOriginal = originalProject.services.get(service);
                if (!toPackageOriginal) {
                    throw new Error(`Cannot package service ${service}. This service does not exist in current project. Make sure project is initialized with yarn install`);
                }
                this._packagePath = (0, path_1.join)(toPackageOriginal.root, '.package');
                if (shouldResolvesServiceNodeModules) {
                    this._logger.debug('Creating transient project...');
                    const transientProject = await this._createTransientProject(originalProject);
                    this._logger.debug('Focusing workspace', toPackageOriginal.name, '...');
                    const dependentWorkspaces = this._focusWorkspace(transientProject, toPackageOriginal);
                    this._logger.debug('Copying compiled code of dependent workspaces');
                    await this._copyCompiledFiles(originalProject, dependentWorkspaces);
                    obs.next({ message: 'Transient project created', took: Date.now() - now });
                    now = Date.now();
                    this._logger.debug('Running yarn install on patched transient project...');
                    await this._yarnInstall(transientProject);
                    obs.next({ message: 'Dependencies resolved', took: Date.now() - now });
                    now = Date.now();
                }
                else {
                    await (0, fs_extra_1.mkdirp)(this._packagePath);
                }
                this._logger.debug('Generating zip file...');
                const megabytes = await this._generateArchives(toPackageOriginal, level);
                obs.next({ message: 'Zip file generated', took: Date.now() - now, megabytes, overall: Date.now() - started });
                if (!this._packagePath) {
                    throw new Error('Assertion failed: package path should have been resolved');
                }
                (0, fs_extra_1.writeJSONSync)((0, path_1.join)(this._packagePath, 'bundle-metadata.json'), { megabytes, took: Date.now() - started });
                this._logger.debug('Removing temporary path');
                this._tmpPath = (0, path_1.join)(this._packagePath, 'tmp');
                await (0, fs_extra_1.remove)(this._tmpPath);
            };
            pkg()
                .then(() => obs.complete())
                .catch((e) => obs.error(e));
        });
    }
    async _createTransientProject(originalProject) {
        if (!this._packagePath) {
            throw new Error('Assertion failed: temporary path should have been resolved');
        }
        this._tmpPath = (0, path_1.join)(this._packagePath, 'tmp');
        this._logger.debug('Creating transient project in', this._tmpPath);
        const doesPackagePathExists = await (0, fs_extra_1.pathExists)(this._tmpPath);
        if (doesPackagePathExists) {
            this._logger.debug('Temporary path already exists, removing it');
            await (0, fs_extra_1.remove)(this._tmpPath);
        }
        await (0, fs_extra_1.mkdirp)(this._tmpPath);
        this._logger.debug('Temporary directory successfully created');
        const manifests = Array.from(originalProject.workspaces.values())
            .filter((w) => !w.root.includes('/tooling/'))
            .map((w) => (0, path_1.join)(w.root, 'package.json'));
        await Promise.all(manifests.map(async (path) => {
            const dest = (0, path_1.join)(String(this._tmpPath), (0, path_1.relative)(this._projectRoot, path));
            const targetDir = (0, path_1.dirname)(dest);
            const exists = await (0, fs_extra_1.pathExists)(targetDir);
            if (!exists) {
                await (0, fs_extra_1.mkdirp)(targetDir);
            }
            this._logger.debug('Copying workspaces manifests', path, '->', dest);
            await (0, fs_extra_1.copyFile)(path, dest);
        }));
        this._logger.debug('Copying yarn lock, cache and config');
        this._logger.debug((0, path_1.join)(this._projectRoot, '.yarnrc.yml'), '->', (0, path_1.join)(this._tmpPath, '.yarnrc.yml'));
        this._logger.debug((0, path_1.join)(this._projectRoot, '.yarn'), '->', (0, path_1.join)(this._tmpPath, '.yarn'));
        this._logger.debug((0, path_1.join)(this._projectRoot, 'yarn.lock'), '->', (0, path_1.join)(this._tmpPath, 'yarn.lock'));
        await (0, fs_extra_1.copyFile)((0, path_1.join)(this._projectRoot, '.yarnrc.yml'), (0, path_1.join)(this._tmpPath, '.yarnrc.yml'));
        await (0, fs_extra_1.copy)((0, path_1.join)(this._projectRoot, 'package.json'), (0, path_1.join)(this._tmpPath, 'package.json'));
        await (0, fs_extra_1.copy)((0, path_1.join)(this._projectRoot, '.yarn'), (0, path_1.join)(this._tmpPath, '.yarn'));
        await (0, fs_extra_1.copy)((0, path_1.join)(this._projectRoot, 'yarn.lock'), (0, path_1.join)(this._tmpPath, 'yarn.lock'));
        this._logger.debug('Verifying transient project...');
        const transientProject = await project_1.Project.loadProject(this._tmpPath);
        this._logger.debug('Transient project workspaces', transientProject.workspaces.keys());
        return transientProject;
    }
    _focusWorkspace(transientProject, toFocus) {
        const toPackageTransient = transientProject.workspaces.get(toFocus.name);
        if (!toPackageTransient) {
            throw new Error(`Assertion failed: Cannot package workspace ${toFocus} in transient project`);
        }
        const manifest = (0, fs_extra_1.readJSONSync)((0, path_1.join)(transientProject.root, 'package.json'));
        delete manifest.devDependencies;
        (0, fs_extra_1.writeJSONSync)((0, path_1.join)(transientProject.root, 'package.json'), manifest, { spaces: 2 });
        this._logger.debug('Find dependencies of workspace to focus', toFocus.name);
        const requiredWorkspaces = new Set([toPackageTransient]);
        const addWorkspacesDependencies = (workspace) => {
            for (const dep of workspace.descendants.values()) {
                const internalDependency = transientProject.workspaces.get(dep.name);
                if (internalDependency) {
                    this._logger.debug('Internal dep', dep.name);
                    requiredWorkspaces.add(internalDependency);
                }
            }
        };
        addWorkspacesDependencies(toPackageTransient);
        this._logger.debug('Dependencies workspaces', Array.from(requiredWorkspaces).map((w) => w.name));
        const isRequired = (workspace) => Array.from(requiredWorkspaces).some((w) => w.name === workspace.name);
        this._logger.debug('Patching manifests');
        for (const workspace of transientProject.workspaces.values()) {
            if (isRequired(workspace)) {
                this._logger.debug(workspace.name, 'is dependency of', toPackageTransient.name, 'removing only devDeps');
                const manifest = (0, fs_extra_1.readJSONSync)((0, path_1.join)(workspace.root, 'package.json'));
                delete manifest.devDependencies;
                (0, fs_extra_1.writeJSONSync)((0, path_1.join)(workspace.root, 'package.json'), manifest, { spaces: 2 });
            }
            else {
                this._logger.debug(workspace.name, 'is not dependency of', toPackageTransient.name, 'removing all dependencies');
                const manifest = (0, fs_extra_1.readJSONSync)((0, path_1.join)(workspace.root, 'package.json'));
                delete manifest.dependencies;
                delete manifest.devDependencies;
                delete manifest.peerDependencies;
                delete manifest.scripts;
                (0, fs_extra_1.writeJSONSync)((0, path_1.join)(workspace.root, 'package.json'), manifest, { spaces: 2 });
            }
        }
        return Array.from(requiredWorkspaces);
    }
    async _copyCompiledFiles(originalProject, workspaces) {
        await Promise.all(workspaces.map(async (w) => {
            this._logger.debug('Copying compiled files of', w.name);
            const originalWorkspace = originalProject.workspaces.get(w.name);
            if (!originalWorkspace) {
                throw new Error(`Assertion failed: originalWorkspace not found`);
            }
            const config = (0, utils_1.getTsConfig)(originalWorkspace.root);
            this._logger.debug('outDir resolved', config?.options?.outDir);
            if (!config?.options?.outDir) {
                throw new Error(`Out directory could be resolve for ${originalWorkspace.name}. Make sure a valid tsconfig.json can be found at package root with a specified outDir`);
            }
            this._logger.debug('Copying', config.options.outDir, '->', (0, path_1.join)(w.root, (0, path_1.relative)(originalWorkspace.root, config.options.outDir)));
            await (0, fs_extra_1.copy)(config.options.outDir, (0, path_1.join)(w.root, (0, path_1.relative)(originalWorkspace.root, config.options.outDir)));
        }));
    }
    async _yarnInstall(project) {
        try {
            await (0, execa_1.command)('yarn install --no-immutable', {
                cwd: project.root,
                stdio: process.env.MILA_DEBUG?.split(',').includes('packagr') ? 'inherit' : 'pipe',
            });
        }
        catch (e) {
            this._logger.error(e);
            throw Error('Yarn install failed in transient project');
        }
    }
    async _generateArchives(toPackageOriginal, level) {
        const megabytes = {
            code: await this._generateCodeArchive(toPackageOriginal, level),
        };
        if (this._useLayers && this._buildLayer) {
            megabytes.layer = await this._generateLayerArchive(level);
        }
        return megabytes;
    }
    async _generateLayerArchive(level) {
        const layerZipName = 'layer.zip';
        const toZip = new Map();
        this._copyDependencies(toZip);
        return this._generateZip(toZip, layerZipName, level);
    }
    _copyDependencies(toZip) {
        if (!this._tmpPath) {
            throw new Error('Assertion failed: temporary path should have been resolved');
        }
        this._logger.debug('Copying node_modules');
        const dependencies = (0, glob_1.sync)((0, path_1.join)(this._tmpPath, 'node_modules', '**', '*'), {
            follow: true,
        });
        dependencies.forEach((path) => {
            this._logger.debug((0, path_1.relative)(String(this._tmpPath), path), '->', path);
            toZip.set((0, path_1.relative)(String(this._tmpPath), path), path);
        });
    }
    _copyCompiledSources(toZip, toPackageOriginal) {
        const tsConfig = (0, utils_1.getTsConfig)(toPackageOriginal.root);
        const lib = tsConfig?.options?.outDir;
        if (!lib) {
            throw new Error(`Out directory could be resolve for ${toPackageOriginal.name}. Make sure a valid tsconfig.json can be found at package root with a specified outDir`);
        }
        this._logger.debug('Copying service to package compiled code. outDir =', lib);
        const compiledSources = (0, glob_1.sync)((0, path_1.join)(lib, '**', '*.js'));
        compiledSources.forEach((js) => {
            this._logger.debug((0, path_1.relative)(lib, js), '->', js);
            toZip.set((0, path_1.relative)(lib, js), js);
        });
    }
    async _generateCodeArchive(toPackageOriginal, level) {
        const zipName = 'bundle.zip';
        const toZip = new Map();
        if (!this._useLayers) {
            this._copyDependencies(toZip);
        }
        this._copyCompiledSources(toZip, toPackageOriginal);
        return this._generateZip(toZip, zipName, level);
    }
    async _generateZip(toZip, zipName, level) {
        return new Promise((resolve, reject) => {
            if (!this._packagePath) {
                throw new Error('Assertion failed: package path should have been resolved');
            }
            this._logger.debug('Opening write stream on', (0, path_1.join)(this._packagePath, zipName));
            const output = (0, fs_1.createWriteStream)((0, path_1.join)(this._packagePath, zipName));
            const archive = (0, archiver_1.default)('zip', {
                zlib: { level },
            });
            output.on('close', () => {
                const megabytes = Math.round(100 * (archive.pointer() / 1000000)) / 100;
                return resolve(megabytes);
            });
            archive.on('warning', (err) => {
                if (err.code === 'ENOENT') {
                }
                else {
                    return reject(err);
                }
            });
            archive.on('error', (err) => {
                return reject(err);
            });
            archive.pipe(output);
            this._logger.debug('Applying correct chmod');
            toZip.forEach((from, dest) => {
                const stats = (0, fs_extra_1.statSync)(from);
                if (stats.isFile()) {
                    archive.file(from, {
                        name: dest,
                        mode: 0o644,
                    });
                }
            });
            this._logger.debug('Finalize');
            archive.finalize();
        });
    }
}
exports.Packager = Packager;
//# sourceMappingURL=packagr.js.map