"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveDeltas = exports.isAffected = void 0;
const utils_1 = require("@microlambda/utils");
const runner_core_1 = require("@microlambda/runner-core");
const remote_state_1 = require("@microlambda/remote-state");
const config_1 = require("@microlambda/config");
const read_config_1 = require("./read-config");
const aws_1 = require("@microlambda/aws");
const readStateConfig = (projectRoot, account) => {
    const rootConfig = (0, read_config_1.readConfig)(projectRoot);
    return (0, config_1.getStateConfig)(rootConfig, account);
};
const isAffected = async (options, logger) => {
    let workspace;
    let projectRoot;
    if (typeof options.workspace === 'string') {
        const getProject = async () => {
            projectRoot = (0, utils_1.resolveProjectRoot)();
            return options.project ?? (await runner_core_1.Project.loadProject(projectRoot));
        };
        const project = options.project ?? (await getProject());
        workspace = project.workspaces.get(options.workspace);
    }
    else {
        workspace = options.workspace;
    }
    logger?.debug('is affected', workspace?.name, options.cmd);
    if (!workspace) {
        throw new Error('Unknown workspace: ' + options.workspace);
    }
    if (!workspace.hasCommand(options.cmd)) {
        logger?.debug('workspace has not the target impelmented');
        return {
            workspace,
            affected: false,
            hasCommand: false,
        };
    }
    let state = options.state;
    logger?.debug({ state });
    if (!state) {
        const projectRoot = options.project?.root ?? (0, utils_1.resolveProjectRoot)();
        const config = options.config ?? readStateConfig(projectRoot, options.account);
        logger?.debug({ config, projectRoot });
        await (0, remote_state_1.verifyState)(config, console);
        logger?.debug('state valid');
        state = options.state ?? new remote_state_1.State(config.state.table, config.defaultRegion);
        logger?.debug('state resolved', config.state.table, config.defaultRegion);
    }
    let sha1;
    let branch;
    if (options.fromRevision.match(/^[0-9a-f]{40}$/)) {
        sha1 = options.fromRevision;
    }
    else {
        const latestBranchExecution = await state.getLatestBranchExecution({
            branch: options.fromRevision,
            cmd: options.cmd,
            args: options.args ?? [],
            env: options.env ?? {},
            workspace: workspace.name,
        });
        sha1 = latestBranchExecution?.sha1;
        if (sha1) {
            branch = options.fromRevision;
        }
    }
    if (!sha1) {
        logger?.debug('no previous execution found: affected = true');
        return {
            workspace,
            affected: true,
            branch,
            hasCommand: true,
        };
    }
    const cachedExecution = await state.getExecution({
        sha1,
        cmd: options.cmd,
        args: options.args ?? [],
        env: options.env ?? {},
        workspace: workspace.name,
    });
    logger?.debug('no previous execution found: affected = true');
    if (!cachedExecution) {
        return {
            workspace,
            affected: true,
            hasCommand: true,
            since: sha1,
            branch,
        };
    }
    const readChecksums = async (execution) => {
        try {
            const raw = await aws_1.aws.s3.downloadBuffer(execution.bucket, execution.checksums, execution.region);
            if (!raw) {
                return {};
            }
            return JSON.parse(raw.toString('utf-8'));
        }
        catch (e) {
            return {};
        }
    };
    const checksums = new runner_core_1.Checksums(workspace, options.cmd, options.args ?? [], options.env ?? {});
    const [currentChecksums, storedChecksum] = await Promise.all([checksums.calculate(), readChecksums(cachedExecution)]);
    logger?.debug('previous execution found', sha1);
    const areEquals = runner_core_1.Checksums.compare(currentChecksums, storedChecksum);
    logger?.debug('comparing cehcksums', { areEquals });
    return {
        workspace,
        affected: !areEquals,
        hasCommand: true,
        since: sha1,
        branch,
        cachedExecution,
    };
};
exports.isAffected = isAffected;
const resolveDeltas = async (options) => {
    const projectRoot = options.project?.root ?? (0, utils_1.resolveProjectRoot)();
    const project = options.project ?? (await runner_core_1.Project.loadProject(projectRoot));
    const config = options.config ?? readStateConfig(projectRoot, options.account);
    await (0, remote_state_1.verifyState)(config);
    const shouldRun = [];
    const state = options.state ?? new remote_state_1.State(config.state.table, config.defaultRegion);
    for (const workspace of project.workspaces.values()) {
        shouldRun.push(await (0, exports.isAffected)({ ...options, workspace, state }));
    }
    return shouldRun;
};
exports.resolveDeltas = resolveDeltas;
//# sourceMappingURL=resolve-deltas.js.map