"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scheduler = void 0;
const rxjs_1 = require("rxjs");
const utils_1 = require("@microlambda/utils");
const runner_core_1 = require("@microlambda/runner-core");
const types_1 = require("@microlambda/types");
const environments_1 = require("@microlambda/environments");
const environment_1 = require("./environment");
class Scheduler {
    project;
    defaultRegion;
    _logger;
    _concurrency;
    _runner;
    _process;
    constructor(project, defaultRegion, logger, concurrency) {
        this.project = project;
        this.defaultRegion = defaultRegion;
        this._logger = logger.scope('core/scheduler');
        this._logger.debug('New recompilation scheduler instance');
        this._concurrency = concurrency || (0, utils_1.getDefaultThreads)();
        this._runner = new runner_core_1.Runner(this.project, this._concurrency, logger);
    }
    startOne(service) {
        this._logger.info('Starting', service.name);
        this._runner.addWorkspaces('start', [service]);
        this._runner.addWorkspaces('build', [service]);
        this._runner.addWorkspaces('transpile', [service]);
    }
    startAll() {
        this._runner.addWorkspaces('start', [...this.project.services.values()]);
        this._runner.addWorkspaces('build', [...this.project.services.values()]);
        this._runner.addWorkspaces('transpile', [...this.project.services.values()]);
    }
    stopOne(service) {
        this._runner.removeWorkspace('start', [service]);
        this._runner.removeWorkspace('build', [service]);
        this._runner.removeWorkspace('transpile', [service]);
    }
    gracefulShutdown() {
        this._runner.removeWorkspace('start', [...this.project.services.values()]);
        this._runner.removeWorkspace('build', [...this.project.services.values()]);
        this._runner.removeWorkspace('transpile', [...this.project.services.values()]);
    }
    stopAll() {
        return this.gracefulShutdown();
    }
    restartOne(service) {
        this.stopOne(service);
        this.startOne(service);
    }
    restartAll() {
        this.stopAll();
        this.startAll();
    }
    get execution$() {
        if (!this._process) {
            throw new Error('No process running');
        }
        return this._process;
    }
    exec(initialScope, debounce) {
        if (this._process) {
            throw new Error('A process is already running');
        }
        this._process = new rxjs_1.Observable((obs) => {
            this._resolveEnvs()
                .then((envs) => {
                const transpile$ = this._runner
                    .runCommand({
                    cmd: 'transpile',
                    mode: 'topological',
                    to: initialScope,
                    watch: true,
                    debounce: debounce.transpile,
                })
                    .pipe((0, rxjs_1.tap)((evt) => {
                    switch (evt.type) {
                        case runner_core_1.RunCommandEventEnum.NODE_STARTED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .transpiled(types_1.TranspilingStatus.TRANSPILING);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_PROCESSED:
                            const workspace = this.project.getWorkspace(evt.target.workspace.name);
                            workspace?.updateStatus().transpiled(types_1.TranspilingStatus.TRANSPILED);
                            workspace?.updateMetric().transpile({
                                took: evt.result.overall,
                                finishedAt: new Date().toISOString(),
                                fromCache: evt.result.fromCache,
                            });
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_ERRORED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .transpiled(types_1.TranspilingStatus.ERROR_TRANSPILING);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_INTERRUPTED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .transpiled(types_1.TranspilingStatus.NOT_TRANSPILED);
                            break;
                    }
                }), (0, rxjs_1.map)((evt) => ({ ...evt, cmd: 'transpile' })));
                const build$ = this._runner
                    .runCommand({
                    cmd: 'build',
                    mode: 'topological',
                    to: initialScope,
                    watch: true,
                    debounce: debounce.build,
                })
                    .pipe((0, rxjs_1.tap)((evt) => {
                    switch (evt.type) {
                        case runner_core_1.RunCommandEventEnum.NODE_STARTED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .typechecked(types_1.TypeCheckStatus.CHECKING);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_PROCESSED:
                            const workspace = this.project.getWorkspace(evt.target.workspace.name);
                            workspace?.updateStatus().typechecked(types_1.TypeCheckStatus.SUCCESS);
                            workspace?.updateMetric().typecheck({
                                took: evt.result.overall,
                                finishedAt: new Date().toISOString(),
                                fromCache: evt.result.fromCache,
                            });
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_ERRORED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .typechecked(types_1.TypeCheckStatus.ERROR);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_INTERRUPTED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .typechecked(types_1.TypeCheckStatus.NOT_CHECKED);
                            break;
                    }
                }), (0, rxjs_1.map)((evt) => ({ ...evt, cmd: 'build' })));
                const start$ = this._runner
                    .runCommand({
                    cmd: 'start',
                    mode: 'parallel',
                    watch: true,
                    force: true,
                    workspaces: initialScope,
                    debounce: debounce.start,
                    args: this._startArgs,
                    releasePorts: this._ports,
                    env: envs,
                })
                    .pipe((0, rxjs_1.tap)((evt) => {
                    switch (evt.type) {
                        case runner_core_1.RunCommandEventEnum.NODE_STARTED:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .started(types_1.ServiceStatus.STARTING);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_PROCESSED:
                            const workspace = this.project.getWorkspace(evt.target.workspace.name);
                            workspace?.updateStatus().started(types_1.ServiceStatus.RUNNING);
                            workspace?.updateMetric().start({
                                took: evt.result.overall,
                                finishedAt: new Date().toISOString(),
                                fromCache: evt.result.fromCache,
                            });
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_ERRORED:
                            this.project.getWorkspace(evt.target.workspace.name)?.updateStatus().started(types_1.ServiceStatus.CRASHED);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_INTERRUPTING:
                            this.project
                                .getWorkspace(evt.target.workspace.name)
                                ?.updateStatus()
                                .started(types_1.ServiceStatus.STOPPING);
                            break;
                        case runner_core_1.RunCommandEventEnum.NODE_INTERRUPTED:
                            this.project.getWorkspace(evt.target.workspace.name)?.updateStatus().started(types_1.ServiceStatus.STOPPED);
                            break;
                    }
                }), (0, rxjs_1.map)((evt) => ({ ...evt, cmd: 'start' })));
                (0, rxjs_1.from)([transpile$, build$, start$])
                    .pipe((0, rxjs_1.mergeAll)())
                    .subscribe({
                    next: (evt) => obs.next(evt),
                    error: (e) => obs.error(e),
                    complete: () => obs.complete(),
                });
            })
                .catch((e) => obs.error(e));
        });
        return this._process;
    }
    get _startArgs() {
        const args = new Map();
        for (const service of this.project.services.values()) {
            if (service.ports && service.isSlsService) {
                args.set(service.name, [
                    `--httpPort ${service.ports?.http.toString() || '3000'} --lambdaPort ${service.ports?.lambda.toString() || '4000'} --websocketPort ${service.ports?.websocket.toString() || '6000'} --reloadHandler`,
                ]);
            }
        }
        return args;
    }
    get _ports() {
        const args = new Map();
        for (const service of this.project.services.values()) {
            if (service.ports) {
                args.set(service.name, Object.values(service.ports));
            }
        }
        return args;
    }
    async _resolveEnvs() {
        return (0, environment_1.resolveEnvs)(this.project, 'local', environments_1.SSMResolverMode.IGNORE, this.defaultRegion, this._logger);
    }
}
exports.Scheduler = Scheduler;
//# sourceMappingURL=scheduler.js.map