"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeSharedInfrastructure = exports.deploySharedInfrastructure = void 0;
const rxjs_1 = require("rxjs");
const types_1 = require("./types");
const stack_deployer_1 = require("./stack-deployer");
const remote_state_1 = require("@microlambda/remote-state");
const _applySharedInfrastructure = async (params, logger) => {
    const { action, env, concurrency, onlyEnvSpecific, project, config, verbose, force, currentRevision } = params;
    return new rxjs_1.Observable((obs) => {
        const sharedInfraWorkspaces = [];
        for (const workspace of project.workspaces.values()) {
            if (workspace.hasCommand('infra:deploy') && workspace.hasCommand('infra:remove')) {
                logger?.debug('Found eligible workspace', workspace.name);
                sharedInfraWorkspaces.push(workspace);
            }
        }
        logger?.debug('Deploying shared infrastructure');
        logger?.debug('Stacks', sharedInfraWorkspaces.map((w) => w.name));
        const state = new remote_state_1.State(config.state.table, config.defaultRegion);
        obs.next({
            type: types_1.SharedInfraDeployEventType.WORKSPACES_RESOLVED,
            workspaces: sharedInfraWorkspaces,
        });
        const deployments$ = sharedInfraWorkspaces
            .filter((workspace) => !onlyEnvSpecific || workspace._config.sharedInfra?.envSpecific)
            .map((workspace) => action === 'deploy'
            ? (0, stack_deployer_1.deploySharedInfraStack)({
                env,
                workspace,
                state,
                config,
                verbose,
                force,
                project,
                concurrency,
                currentRevision,
            }, logger)
            : (0, stack_deployer_1.removeSharedInfraStack)({
                env,
                workspace,
                state,
                config,
                verbose,
                project,
                concurrency,
                currentRevision,
            }, logger));
        logger?.debug('Prepared', deployments$.length, 'deploy/remove operations');
        (0, rxjs_1.from)(deployments$)
            .pipe((0, rxjs_1.mergeAll)())
            .subscribe({
            next: (evt) => obs.next(evt),
            error: (err) => obs.error(err),
            complete: () => {
                logger?.debug('All operations completed');
                obs.complete();
            },
        });
    });
};
const deploySharedInfrastructure = async (params, logger) => {
    return _applySharedInfrastructure({ action: 'deploy', ...params }, logger);
};
exports.deploySharedInfrastructure = deploySharedInfrastructure;
const removeSharedInfrastructure = async (params, logger) => {
    return _applySharedInfrastructure({ action: 'remove', ...params, force: true }, logger);
};
exports.removeSharedInfrastructure = removeSharedInfrastructure;
//# sourceMappingURL=index.js.map