"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeSharedInfraStack = exports.deploySharedInfraStack = void 0;
const rxjs_1 = require("rxjs");
const types_1 = require("./types");
const runner_core_1 = require("@microlambda/runner-core");
const runSlsCommand = (params, logger) => {
    const { action, region, verbose, env, workspace, force, currentRevision, state, config, project, concurrency } = params;
    return new rxjs_1.Observable((obs) => {
        logger?.debug('starting evt', workspace.name, env, region, action);
        obs.next({
            type: action === 'deploy' ? types_1.SharedInfraDeployEventType.DEPLOYING : types_1.SharedInfraDeployEventType.REMOVING,
            region,
            env: env,
            workspace,
        });
        let updatedState$;
        const _env = workspace._config.sharedInfra?.envSpecific ? env : undefined;
        const cachePrefix = workspace._config.sharedInfra?.envSpecific
            ? `caches/${workspace.name}/${action}/${region}`
            : `caches/${workspace.name}/${action}/${env}/${region}`;
        const runner = new runner_core_1.Runner(project, concurrency);
        runner
            .runCommand({
            cmd: `infra:${action}`,
            mode: 'parallel',
            workspaces: [workspace],
            env: {
                ...process.env,
                AWS_REGION: region,
                ENV: env,
            },
            force: action === 'remove' || force,
            stdio: verbose ? 'inherit' : 'pipe',
            remoteCache: action === 'deploy'
                ? {
                    region: config.defaultRegion,
                    bucket: config.state.checksums,
                    table: config.state.table,
                }
                : undefined,
            cachePrefix,
        })
            .subscribe({
            next: (evt) => {
                switch (evt.type) {
                    case runner_core_1.RunCommandEventEnum.NODE_STARTED:
                        obs.next({
                            type: action === 'deploy' ? types_1.SharedInfraDeployEventType.DEPLOYING : types_1.SharedInfraDeployEventType.REMOVING,
                            region,
                            env: env,
                            workspace,
                        });
                        break;
                    case runner_core_1.RunCommandEventEnum.NODE_SKIPPED:
                        obs.next({
                            type: types_1.SharedInfraDeployEventType.NO_CHANGES,
                            region,
                            env: env,
                            workspace,
                        });
                        break;
                    case runner_core_1.RunCommandEventEnum.NODE_PROCESSED:
                        logger?.debug('Node processed', action, workspace.name, region);
                        const request = {
                            name: workspace.name,
                            region,
                            sha1: currentRevision,
                            checksums_buckets: config.state.checksums,
                            checksums_key: `${cachePrefix}/${currentRevision}/checksums.json`,
                        };
                        if (_env) {
                            request.env = _env;
                        }
                        updatedState$ =
                            action === 'deploy'
                                ? state.setSharedInfrastructureState(request)
                                : state.deleteSharedInfrastructureState(workspace.name, region, env);
                        obs.next({
                            type: action === 'deploy' ? types_1.SharedInfraDeployEventType.DEPLOYED : types_1.SharedInfraDeployEventType.REMOVED,
                            region,
                            env: env,
                            workspace,
                            result: evt.result,
                        });
                        break;
                    case runner_core_1.RunCommandEventEnum.NODE_ERRORED:
                        obs.next({
                            type: action === 'deploy'
                                ? types_1.SharedInfraDeployEventType.FAILED_DEPLOY
                                : types_1.SharedInfraDeployEventType.FAILED_REMOVE,
                            region,
                            env: env,
                            workspace,
                            err: evt.error,
                        });
                        break;
                }
            },
            error: (err) => {
                obs.next({
                    type: action === 'deploy' ? types_1.SharedInfraDeployEventType.FAILED_DEPLOY : types_1.SharedInfraDeployEventType.FAILED_REMOVE,
                    region,
                    env: env,
                    workspace,
                    err,
                });
            },
            complete: () => {
                const complete = () => {
                    logger?.debug('Target run', `infra:${action}`, workspace.name, region);
                    obs.complete();
                };
                if (updatedState$) {
                    updatedState$
                        .catch((e) => {
                        logger?.warn('Error updating state');
                        logger?.warn(e);
                    })
                        .finally(() => {
                        logger?.debug('State updated');
                        complete();
                    });
                }
                else {
                    complete();
                }
            },
        });
    });
};
const deploySharedInfraStack = (params, logger) => {
    const { env, workspace, state, config, verbose, force, currentRevision, concurrency, project } = params;
    logger?.debug('Preparing update for workspace', workspace.name, env.name);
    return new rxjs_1.Observable((obs) => {
        const _env = workspace._config.sharedInfra?.envSpecific ? env.name : undefined;
        state
            .getSharedInfrastructureState(workspace.name, _env)
            .then((sharedInfraState) => {
            logger?.debug(sharedInfraState);
            const regionalDeployment$ = [];
            for (const region of env.regions) {
                logger?.debug('Resolving operations for', workspace.name, env.name, region);
                regionalDeployment$.push(runSlsCommand({
                    workspace,
                    config,
                    env: env.name,
                    action: 'deploy',
                    region,
                    state,
                    verbose,
                    force,
                    project,
                    concurrency,
                    currentRevision,
                }, logger));
            }
            const allRegion = new Set();
            for (const region of sharedInfraState.map((s) => s.region)) {
                allRegion.add(region);
            }
            logger?.debug('All region found in state', allRegion);
            for (const region of allRegion) {
                if (!env.regions.includes(region)) {
                    logger?.debug('Environment', env.name, 'is no more replicated in', region);
                    regionalDeployment$.push(runSlsCommand({
                        workspace,
                        config,
                        env: env.name,
                        action: 'remove',
                        region,
                        state,
                        verbose,
                        force: true,
                        project,
                        concurrency,
                        currentRevision,
                    }, logger));
                }
            }
            (0, rxjs_1.from)(regionalDeployment$)
                .pipe((0, rxjs_1.concatAll)())
                .subscribe({
                next: (evt) => obs.next(evt),
                error: (err) => obs.error(err),
                complete: () => {
                    logger?.debug('Target run in all region', workspace.name);
                    obs.complete();
                },
            });
        })
            .catch((err) => obs.error(err));
    });
};
exports.deploySharedInfraStack = deploySharedInfraStack;
const removeSharedInfraStack = (params, logger) => {
    const { env, workspace, state, config, verbose, currentRevision, concurrency, project } = params;
    logger?.debug('Preparing update for stack', workspace.name, env.name);
    const regionalDeployment$ = [];
    for (const region of env.regions) {
        regionalDeployment$.push(runSlsCommand({
            workspace,
            config,
            env: env.name,
            action: 'remove',
            region,
            state,
            verbose,
            force: true,
            project,
            concurrency,
            currentRevision,
        }, logger));
    }
    return (0, rxjs_1.from)(regionalDeployment$).pipe((0, rxjs_1.concatAll)());
};
exports.removeSharedInfraStack = removeSharedInfraStack;
//# sourceMappingURL=stack-deployer.js.map